@@ -1,5 +1,4 @@
 # pricing_models/indices.py
-# -*- coding: utf-8 -*-
 """
 Index factory for QuantLib (identifier-driven).
 
@@ -23,20 +22,21 @@ Notes
 from __future__ import annotations
 
 import datetime
-from typing import Dict, Tuple, Union, Optional
+from collections.abc import Callable, Mapping
+from functools import cache, lru_cache
+from typing import Any
 
 import QuantLib as ql
-from functools import lru_cache
 
+from mainsequence.client import Constant as _C
 from mainsequence.instruments.data_interface import data_interface
 from mainsequence.instruments.utils import to_py_date, to_ql_date
-from mainsequence.client import Constant as _C
 
 # ----------------------------- Cache (ONLY by identifier + date) ----------------------------- #
 
 # key: (index_identifier, target_date_py)
-_IndexCacheKey = Tuple[str, datetime.date]
-_INDEX_CACHE: Dict[_IndexCacheKey, ql.Index] = {}
+_IndexCacheKey = tuple[str, datetime.date]
+_INDEX_CACHE: dict[_IndexCacheKey, ql.Index] = {}
 
 
 def clear_index_cache() -> None:
@@ -47,99 +47,154 @@ def clear_index_cache() -> None:
 # Put every supported identifier here with its curve + index construction config.
 # No tenor tokens; we store the QuantLib Period directly.
 
-INDEX_CONFIGS: Dict[str, Dict] = {
-    _C.get_value(name="REFERENCE_RATE__TIIE_28"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__VALMER_TIIE_28"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),
-        currency=(ql.MXNCurrency() if hasattr(ql, "MXNCurrency") else ql.USDCurrency()),
-        period=ql.Period(28, ql.Days),
-        settlement_days=1,
-        bdc=ql.ModifiedFollowing,
-        end_of_month=False,
-    ),
-    _C.get_value(name="REFERENCE_RATE__TIIE_91"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__VALMER_TIIE_28"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),
-        currency=ql.MXNCurrency(),
-        period=ql.Period(91, ql.Days),
-        settlement_days=1,
-        bdc=ql.ModifiedFollowing,
-        end_of_month=False,
-    ),
-    _C.get_value(name="REFERENCE_RATE__TIIE_182"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__VALMER_TIIE_28"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),
-        currency=ql.MXNCurrency(),
-        period=ql.Period(182, ql.Days),
-        settlement_days=1,
-        bdc=ql.ModifiedFollowing,
-        end_of_month=False,
-    ),
-    # Add more identifiers here as needed.
-    _C.get_value(name="REFERENCE_RATE__TIIE_OVERNIGHT"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__VALMER_TIIE_28"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),
-        currency=ql.MXNCurrency(),
-        period=ql.Period(1, ql.Days),
-        settlement_days=1,
-        bdc=ql.ModifiedFollowing,
-        end_of_month=False,
-    ),
-    _C.get_value(name="REFERENCE_RATE__CETE_28"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__BANXICO_M_BONOS_OTR"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),  # BONOS accrue on Act/360
-        currency=ql.MXNCurrency(),
-        period=ql.Period(28, ql.Days),  # Coupons every 28 days
-        settlement_days=1,  # T+1 in Mexico since May 27–28, 2024
-        bdc=ql.Following,  # “next banking business day” => Following
-        end_of_month=False,  # Irrelevant when scheduling by days
-    ),
-    _C.get_value(name="REFERENCE_RATE__CETE_91"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__BANXICO_M_BONOS_OTR"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),  # BONOS accrue on Act/360
-        currency=ql.MXNCurrency(),
-        period=ql.Period(91, ql.Days),  # Coupons every 28 days
-        settlement_days=1,  # T+1 in Mexico since May 27–28, 2024
-        bdc=ql.Following,  # “next banking business day” => Following
-        end_of_month=False,  # Irrelevant when scheduling by days
-    ),
+# ----------------------------- Lazy + memoized constant resolver --------------------------- #
 
-    _C.get_value(name="REFERENCE_RATE__CETE_182"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__BANXICO_M_BONOS_OTR"),
-        calendar=(ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()),
-        day_counter=ql.Actual360(),  # BONOS accrue on Act/360
-        currency=ql.MXNCurrency(),
-        period=ql.Period(182, ql.Days),  # Coupons every 182 days
-        settlement_days=1,  # T+1 in Mexico since May 27–28, 2024
-        bdc=ql.Following,  # “next banking business day” => Following
-        end_of_month=False,  # Irrelevant when scheduling by days
-    ),
+@cache
+def _const(name: str) -> str:
+    """Resolve a named constant (UID) once per process."""
+    return _C.get_value(name=name)
 
 
-    _C.get_value(name="REFERENCE_RATE__USD_SOFR"): dict(
-        curve_uid=_C.get_value(name="ZERO_CURVE__UST_CMT_ZERO_CURVE_UID"),
+# ----------------------------- Tiny helpers (keep it DRY) --------------------------------- #
+
+def _mx_calendar() -> ql.Calendar:
+    return ql.Mexico() if hasattr(ql, "Mexico") else ql.TARGET()
+
+def _mx_currency() -> ql.Currency:
+    return ql.MXNCurrency() if hasattr(ql, "MXNCurrency") else ql.USDCurrency()
+
+
+# ----------------------------- Config templates (no API calls here) ----------------------- #
+# Each template is a factory returning the final dict when invoked.
+
+def _tiie(period_days: int) -> Callable[[], dict[str, Any]]:
+    def factory() -> dict[str, Any]:
+        return dict(
+            curve_uid=_const("ZERO_CURVE__VALMER_TIIE_28"),
+            calendar=_mx_calendar(),
+            day_counter=ql.Actual360(),
+            currency=_mx_currency(),
+            period=ql.Period(period_days, ql.Days),
+            settlement_days=1,
+            bdc=ql.ModifiedFollowing,
+            end_of_month=False,
+        )
+    return factory
+
+def _cete(period_days: int) -> Callable[[], dict[str, Any]]:
+    def factory() -> dict[str, Any]:
+        return dict(
+            curve_uid=_const("ZERO_CURVE__BANXICO_M_BONOS_OTR"),
+            calendar=_mx_calendar(),
+            day_counter=ql.Actual360(),     # BONOS accrue on Act/360
+            currency=_mx_currency(),
+            period=ql.Period(period_days, ql.Days),
+            settlement_days=1,              # T+1
+            bdc=ql.Following,               # “next banking business day”
+            end_of_month=False,             # Irrelevant for day-based schedules
+        )
+    return factory
+
+_INDEX_TEMPLATES: dict[str, Callable[[], dict[str, Any]]] = {
+    # Symbolic name  -> factory (resolved lazily)
+    "REFERENCE_RATE__TIIE_28": _tiie(28),
+    "REFERENCE_RATE__TIIE_91": _tiie(91),
+    "REFERENCE_RATE__TIIE_182": _tiie(182),
+    "REFERENCE_RATE__TIIE_OVERNIGHT": _tiie(1),
+
+    "REFERENCE_RATE__CETE_28": _cete(28),
+    "REFERENCE_RATE__CETE_91": _cete(91),
+    "REFERENCE_RATE__CETE_182": _cete(182),
+
+    "REFERENCE_RATE__USD_SOFR": lambda: dict(
+        curve_uid=_const("ZERO_CURVE__UST_CMT_ZERO_CURVE_UID"),
         calendar=ql.UnitedStates(ql.UnitedStates.FederalReserve),
         day_counter=ql.Actual360(),
         currency=ql.USDCurrency(),
-        period=ql.Period(6, ql.Months),  # Semiannual coupons
-        settlement_days=1,  # T+1
+        period=ql.Period(6, ql.Months),   # Semiannual coupons
+        settlement_days=1,                # T+1
         bdc=ql.ModifiedFollowing,
-        end_of_month=False,  # Irrelevant when scheduling by days
+        end_of_month=False,
     ),
 }
 
 
+# ----------------------------- Lazy, dict-like view over configs -------------------------- #
+
+class _LazyIndexConfigs(Mapping[str, dict[str, Any]]):
+    """
+    Read-only mapping that resolves entries on demand and caches results.
+
+    - External keys (Mapping interface): resolved UIDs (the values of _C.get_value(...)).
+    - Convenience access: .get_by_name("REFERENCE_RATE__...") also supported.
+    - Iteration materializes all entries once (still avoids import-time API calls).
+    """
+    def __init__(self, templates: dict[str, Callable[[], dict[str, Any]]]):
+        self._templates = templates
+        self._uid_by_name: dict[str, str] = {}      # symbolic name -> UID
+        self._cfg_by_uid: dict[str, dict[str, Any]] = {}
+
+    def _materialize_by_name(self, name: str) -> tuple[str, dict[str, Any]]:
+        # Resolve UID once
+        uid = self._uid_by_name.get(name)
+        if uid is None:
+            uid = _const(name)                       # single API call per name (memoized)
+            self._uid_by_name[name] = uid
+
+        # Build config once
+        cfg = self._cfg_by_uid.get(uid)
+        if cfg is None:
+            cfg = self._templates[name]()            # may resolve other constants (also memoized)
+            self._cfg_by_uid[uid] = cfg
+        return uid, cfg
+
+    # ----- Public helpers -----
+
+    def get_by_name(self, name: str) -> dict[str, Any]:
+        """Fetch config by symbolic name (no need to call _C.get_value at site)."""
+        _, cfg = self._materialize_by_name(name)
+        return cfg
+
+    # ----- Mapping interface (keys are UIDs) -----
+
+    def __getitem__(self, uid: str) -> dict[str, Any]:
+        # Fast path: already cached by UID
+        cfg = self._cfg_by_uid.get(uid)
+        if cfg is not None:
+            return cfg
+
+        # Otherwise, resolve templates until we find the matching UID
+        for name in self._templates:
+            # Skip names that already resolved to a different UID
+            known_uid = self._uid_by_name.get(name)
+            if known_uid is not None:
+                if known_uid == uid:
+                    return self._cfg_by_uid[uid]
+                continue
+            resolved_uid, cfg = self._materialize_by_name(name)
+            if resolved_uid == uid:
+                return cfg
+        raise KeyError(uid)
+
+    def __iter__(self):
+        # Realize all on-demand for full iteration
+        for name in self._templates:
+            self._materialize_by_name(name)
+        return iter(self._cfg_by_uid)
+
+    def __len__(self) -> int:
+        return len(self._templates)
+
+
+# ----------------------------- Exported (drop-in) mapping --------------------------------- #
+
+INDEX_CONFIGS = _LazyIndexConfigs(_INDEX_TEMPLATES)
+
+
 # ----------------------------- Utilities ----------------------------- #
 
-def _ensure_py_date(
-    d: Union[datetime.date, datetime.datetime, ql.Date]
-) -> datetime.date:
+
+def _ensure_py_date(d: datetime.date | datetime.datetime | ql.Date) -> datetime.date:
     """Return a Python date; target_date is REQUIRED and must not be None."""
     if d is None:
         raise ValueError("target_date is required and cannot be None.")
@@ -153,8 +208,9 @@ def _ensure_py_date(
 
 # ----------------------------- Zero-curve builder (kept) ----------------------------- #
 
+
 def build_zero_curve(
-    target_date: Union[datetime.date, datetime.datetime],
+    target_date: datetime.date | datetime.datetime,
     index_identifier: str,
 ) -> ql.YieldTermStructureHandle:
     """
@@ -175,8 +231,10 @@ def build_zero_curve(
     nodes = data_interface.get_historical_discount_curve(curve_uid, target_date)
 
     base = to_ql_date(target_date)
-    base_py = target_date if isinstance(target_date, datetime.datetime) else datetime.datetime.combine(
-        target_date, datetime.time()
+    base_py = (
+        target_date
+        if isinstance(target_date, datetime.datetime)
+        else datetime.datetime.combine(target_date, datetime.time())
     )
 
     dates = [base]
@@ -212,19 +270,24 @@ def build_zero_curve(
 
 
 @lru_cache(maxsize=256)
-def _default_curve_cached(index_identifier: str, date_key: datetime.date) -> ql.YieldTermStructureHandle:
+def _default_curve_cached(
+    index_identifier: str, date_key: datetime.date
+) -> ql.YieldTermStructureHandle:
     """Small cache for default curves, keyed only by (identifier, date)."""
     target_dt = datetime.datetime.combine(date_key, datetime.time())
     return build_zero_curve(target_dt, index_identifier)
 
 
-def _default_curve(index_identifier: str, target_date: Union[datetime.date, datetime.datetime, ql.Date]) -> ql.YieldTermStructureHandle:
+def _default_curve(
+    index_identifier: str, target_date: datetime.date | datetime.datetime | ql.Date
+) -> ql.YieldTermStructureHandle:
     dk = _ensure_py_date(target_date)
     return _default_curve_cached(index_identifier, dk)
 
 
 # ----------------------------- Historical fixings hydration ----------------------------- #
 
+
 def add_historical_fixings(target_date: ql.Date, ibor_index: ql.IborIndex):
     """
     Backfill historical fixings for an index up to (but not including) target_date,
@@ -240,9 +303,7 @@ def add_historical_fixings(target_date: ql.Date, ibor_index: ql.IborIndex):
     uid = ibor_index.familyName()
 
     historical_fixings = data_interface.get_historical_fixings(
-        reference_rate_uid=uid,
-        start_date=start_date,
-        end_date=end_date
+        reference_rate_uid=uid, start_date=start_date, end_date=end_date
     )
 
     if not historical_fixings:
@@ -268,19 +329,20 @@ def add_historical_fixings(target_date: ql.Date, ibor_index: ql.IborIndex):
 
 # ----------------------------- Index construction ----------------------------- #
 
-def _make_index_from_config(index_identifier: str,
-                            curve: ql.YieldTermStructureHandle,
-                            *,
-                            override_settlement_days: Optional[int] = None) -> ql.IborIndex:
+
+def _make_index_from_config(
+    index_identifier: str,
+    curve: ql.YieldTermStructureHandle,
+    *,
+    override_settlement_days: int | None = None,
+) -> ql.IborIndex:
     """
     Build a ql.IborIndex using the exact spec stored in INDEX_CONFIGS[index_identifier].
     No tenor tokens. Period comes from config.
     """
     cfg = INDEX_CONFIGS.get(index_identifier)
     if cfg is None:
-        raise KeyError(
-            f"No index config for {index_identifier!r}. Add an entry to INDEX_CONFIGS."
-        )
+        raise KeyError(f"No index config for {index_identifier!r}. Add an entry to INDEX_CONFIGS.")
 
     cal: ql.Calendar = cfg["calendar"]
     ccy: ql.Currency = cfg["currency"]
@@ -288,31 +350,26 @@ def _make_index_from_config(index_identifier: str,
     period: ql.Period = cfg["period"]
     bdc: ql.BusinessDayConvention = cfg["bdc"]
     eom: bool = cfg["end_of_month"]
-    settle: int = override_settlement_days if override_settlement_days is not None else cfg["settlement_days"]
+    settle: int = (
+        override_settlement_days if override_settlement_days is not None else cfg["settlement_days"]
+    )
 
     # IMPORTANT: we set the QuantLib index **name** to the UID
     return ql.IborIndex(
-        index_identifier,  # name == UID
-        period,
-        settle,
-        ccy,
-        cal,
-        bdc,
-        eom,
-        dc,
-        curve
+        index_identifier, period, settle, ccy, cal, bdc, eom, dc, curve  # name == UID
     )
 
 
 # ----------------------------- Public API ----------------------------- #
 
+
 def get_index(
-        index_identifier: str,
-        target_date: Union[datetime.date, datetime.datetime, ql.Date],
-        *,
-        forwarding_curve: Optional[ql.YieldTermStructureHandle] = None,
-        hydrate_fixings: bool = True,
-        settlement_days: Optional[int] = None
+    index_identifier: str,
+    target_date: datetime.date | datetime.datetime | ql.Date,
+    *,
+    forwarding_curve: ql.YieldTermStructureHandle | None = None,
+    hydrate_fixings: bool = True,
+    settlement_days: int | None = None,
 ) -> ql.Index:
     """
     Return a QuantLib index instance based ONLY on a stable index_identifier and a target_date.
@@ -352,9 +409,7 @@ def get_index(
 
     # Build the index exactly as configured
     idx = _make_index_from_config(
-        index_identifier=index_identifier,
-        curve=use_curve,
-        override_settlement_days=settlement_days
+        index_identifier=index_identifier, curve=use_curve, override_settlement_days=settlement_days
     )
 
     # Optional: hydrate fixings up to (but not including) target_date
