@@ -1,41 +1,37 @@
 import copy
-
-import pytz
-from typing import Union, Dict, List, Literal, Optional,Tuple, Any
-import pandas as pd
-import numpy as np
 import datetime
-import pandas_market_calendars as mcal
-
-from mainsequence.tdag.data_nodes import DataNode, WrapperDataNode, APIDataNode
-from mainsequence.client import (CONSTANTS, LocalTimeSeriesDoesNotExist, DataNodeUpdate, DynamicTableDataSource,
-                                  UpdateStatistics, AssetCategory, AssetTranslationTable, AssetTranslationRule, AssetFilter
-                                 )
-from mainsequence.client import MARKETS_CONSTANTS, ExecutionVenue
-from mainsequence.client import  DoesNotExist, Asset
-from mainsequence.tdag.data_nodes.utils import (
-    string_frequency_to_minutes,
-    string_freq_to_time_delta,
-)
-import os
-
-from tqdm import tqdm
-from joblib import Parallel, delayed
-
-from mainsequence.virtualfundbuilder.models import AssetsConfiguration
-from mainsequence.virtualfundbuilder.utils import logger, TIMEDELTA
-from typing import Optional
 from functools import lru_cache
 
+import numpy as np
+import pandas as pd
+import pandas_market_calendars as mcal
+import pytz
+from joblib import Parallel, delayed
+from tqdm import tqdm
+
+from mainsequence.client import (
+    Asset,
+    AssetCategory,
+    AssetTranslationTable,
+)
+from mainsequence.tdag.data_nodes import DataNode, WrapperDataNode
+from mainsequence.tdag.data_nodes.utils import (
+    string_freq_to_time_delta,
+    string_frequency_to_minutes,
+)
+from mainsequence.virtualfundbuilder.models import AssetsConfiguration
+from mainsequence.virtualfundbuilder.utils import TIMEDELTA
 
 FULL_CALENDAR = "24/7"
 
+
 @lru_cache(maxsize=256)
 def _get_calendar_by_name(calendar_name: str):
     if calendar_name == FULL_CALENDAR:
         return None
     return mcal.get_calendar(calendar_name)
 
+
 @lru_cache(maxsize=1024)
 def _get_schedule_cached(calendar_name: str, start_date: str, end_date: str) -> pd.DataFrame:
     """
@@ -46,31 +42,35 @@ def _get_schedule_cached(calendar_name: str, start_date: str, end_date: str) ->
     if cal is None:
         # 24/7 synthetic schedule (open at 00:00, close 23:59 UTC)
         sessions = pd.date_range(start=start_date, end=end_date, freq="D", tz="UTC")
-        df = pd.DataFrame({
-            "market_open": sessions,
-            "market_close": sessions + pd.Timedelta(days=1) - pd.Timedelta(minutes=1),
-        }, index=sessions)
+        df = pd.DataFrame(
+            {
+                "market_open": sessions,
+                "market_close": sessions + pd.Timedelta(days=1) - pd.Timedelta(minutes=1),
+            },
+            index=sessions,
+        )
         df.index.name = "session"
         return df
 
-    sched = cal.schedule(start_date=start_date, end_date=end_date).reset_index().rename(columns={"index": "session"})
-    sched["session"]      = pd.to_datetime(sched["session"], utc=True).dt.normalize()
-    sched["market_open"]  = pd.to_datetime(sched["market_open"], utc=True)
+    sched = (
+        cal.schedule(start_date=start_date, end_date=end_date)
+        .reset_index()
+        .rename(columns={"index": "session"})
+    )
+    sched["session"] = pd.to_datetime(sched["session"], utc=True).dt.normalize()
+    sched["market_open"] = pd.to_datetime(sched["market_open"], utc=True)
     sched["market_close"] = pd.to_datetime(sched["market_close"], utc=True)
     return sched.set_index("session")
 
 
-
-
-
-def get_interpolated_prices_timeseries(assets_configuration: Optional[AssetsConfiguration]=None, asset_list=None,
-
-                                     ):
+def get_interpolated_prices_timeseries(
+    assets_configuration: AssetsConfiguration | None = None,
+    asset_list=None,
+):
     """
     Creates a Wrapper Timeseries for an asset configuration.
     """
 
-
     if assets_configuration is None:
         assert asset_list is not None, "asset_list and assets_configuration both cant be None"
     if assets_configuration is not None:
@@ -82,51 +82,52 @@ def get_interpolated_prices_timeseries(assets_configuration: Optional[AssetsConf
         if asset_list is None:
             return InterpolatedPrices(
                 asset_category_unique_id=assets_configuration.assets_category_unique_id,
-                **prices_configuration_kwargs
+                **prices_configuration_kwargs,
             )
         else:
 
-            return InterpolatedPrices(
-                asset_list=asset_list,
-                **prices_configuration_kwargs
-            )
+            return InterpolatedPrices(asset_list=asset_list, **prices_configuration_kwargs)
 
     raise Exception("Not implemented prices_configuration Kwargs")
 
+
 class UpsampleAndInterpolation:
     """
     Handles upsampling and interpolation of bar data.
     """
+
     TIMESTAMP_COLS = ["first_trade_time", "last_trade_time", "open_time"]
 
     def __init__(
-            self,
-            bar_frequency_id: str,
-            upsample_frequency_id: str,
-            intraday_bar_interpolation_rule: str,
+        self,
+        bar_frequency_id: str,
+        upsample_frequency_id: str,
+        intraday_bar_interpolation_rule: str,
     ):
         self.bar_frequency_id = bar_frequency_id
         self.upsample_frequency_id = upsample_frequency_id
         self.intraday_bar_interpolation_rule = intraday_bar_interpolation_rule
 
-        rows = string_frequency_to_minutes(self.upsample_frequency_id) / string_frequency_to_minutes(
-            self.bar_frequency_id)
+        rows = string_frequency_to_minutes(
+            self.upsample_frequency_id
+        ) / string_frequency_to_minutes(self.bar_frequency_id)
         assert rows.is_integer()
 
         if "d" in self.bar_frequency_id:
-            assert bar_frequency_id == self.upsample_frequency_id  # Upsampling for daily bars not implemented
+            assert (
+                bar_frequency_id == self.upsample_frequency_id
+            )  # Upsampling for daily bars not implemented
 
         self.upsample_frequency_td = string_freq_to_time_delta(self.upsample_frequency_id)
 
-
     @staticmethod
     def upsample_bars(
-            bars_df: pd.DataFrame,
-            upsample_frequency_obs: int,
-            upsample_frequency_td: object,
-            calendar: str,
-            open_to_close_time_delta: datetime.timedelta,
-            is_portfolio: bool = False
+        bars_df: pd.DataFrame,
+        upsample_frequency_obs: int,
+        upsample_frequency_td: object,
+        calendar: str,
+        open_to_close_time_delta: datetime.timedelta,
+        is_portfolio: bool = False,
     ) -> pd.DataFrame:
         """
         Upsamples the bars dataframe based on the given parameters.
@@ -150,18 +151,19 @@ class UpsampleAndInterpolation:
 
         trading_halts = calendar != FULL_CALENDAR
 
-
-
-
         calendar = mcal.get_calendar(calendar)
 
-        full_schedule = calendar.schedule(bars_df["trade_day"].min(), bars_df["trade_day"].max()).reset_index()
+        full_schedule = calendar.schedule(
+            bars_df["trade_day"].min(), bars_df["trade_day"].max()
+        ).reset_index()
         full_schedule["index"] = full_schedule["index"].apply(lambda x: x.timestamp())
         full_schedule = full_schedule.set_index("index").to_dict("index")
 
         all_dfs = []
-        for i in tqdm(range(bars_df.shape[0] - upsample_frequency_obs + 1),
-                      desc=f"Upsampling from {bars_df['trade_day'].iloc[0]} to {bars_df['trade_day'].iloc[-1]} for assets {bars_df['unique_identifier'].dropna().unique()}"):
+        for i in tqdm(
+            range(bars_df.shape[0] - upsample_frequency_obs + 1),
+            desc=f"Upsampling from {bars_df['trade_day'].iloc[0]} to {bars_df['trade_day'].iloc[-1]} for assets {bars_df['unique_identifier'].dropna().unique()}",
+        ):
             start = i
             end = i + upsample_frequency_obs
             tmp_df = bars_df.iloc[start:end]
@@ -190,10 +192,12 @@ class UpsampleAndInterpolation:
                     "close": close,
                 }
                 if not is_portfolio:
-                    new_bar.update({
-                        "high": np.nanmax(tmp_df.high.values),
-                        "low": np.nanmin(tmp_df.low.values),
-                    })
+                    new_bar.update(
+                        {
+                            "high": np.nanmax(tmp_df.high.values),
+                            "low": np.nanmin(tmp_df.low.values),
+                        }
+                    )
 
             all_dfs.append(new_bar)
 
@@ -203,12 +207,8 @@ class UpsampleAndInterpolation:
 
         return all_dfs
 
-
     def get_interpolated_upsampled_bars(
-            self,
-            calendar: str,
-            tmp_df: pd.DataFrame,
-            last_observation: Union[None, pd.Series] = None
+        self, calendar: str, tmp_df: pd.DataFrame, last_observation: None | pd.Series = None
     ) -> pd.DataFrame:
         """
         Gets interpolated and upsampled bars based on the given parameters.
@@ -266,7 +266,9 @@ class UpsampleAndInterpolation:
             all_columns = self.TIMESTAMP_COLS
             upsampled_df = tmp_df
         else:
-            upsample_freq_obs = string_frequency_to_minutes(self.upsample_frequency_id) // bars_frequency_min
+            upsample_freq_obs = (
+                string_frequency_to_minutes(self.upsample_frequency_id) // bars_frequency_min
+            )
 
             if upsample_freq_obs > bars_frequency_min:
                 upsampled_df = UpsampleAndInterpolation.upsample_bars(
@@ -289,10 +291,10 @@ class UpsampleAndInterpolation:
 
 
 def interpolate_daily_bars(
-        bars_df: pd.DataFrame,
-        interpolation_rule: str,
-        calendar: str,
-        last_observation: Union[None, pd.Series] = None, #fix annotation
+    bars_df: pd.DataFrame,
+    interpolation_rule: str,
+    calendar: str,
+    last_observation: None | pd.Series = None,  # fix annotation
 ):
     try:
         calendar_name = getattr(calendar, "name", calendar)
@@ -307,9 +309,9 @@ def interpolate_daily_bars(
 
                 bars_df = pd.concat([last_observation, bars_df], axis=0).sort_index()
                 if "unique_identifier" in bars_df.columns:
-                    bars_df.loc[:, ['unique_identifier']] = bars_df[
-                        ['unique_identifier']
-                    ].bfill().ffill()
+                    bars_df.loc[:, ["unique_identifier"]] = (
+                        bars_df[["unique_identifier"]].bfill().ffill()
+                    )
 
             null_index = bars_df.isnull().any(axis=1)
             bars_df["close"] = bars_df["close"].ffill()
@@ -341,7 +343,6 @@ def interpolate_daily_bars(
 
         return bars_df
 
-
     # Extend the schedule window so midnight stamps on non-trading days can fill to the next session close
     start_date = bars_df.index.min().date().isoformat()
     end_date = (pd.Timestamp(bars_df.index.max()).normalize()).date().isoformat()
@@ -382,11 +383,11 @@ def interpolate_daily_bars(
 
 
 def interpolate_intraday_bars(
-        bars_df: pd.DataFrame,
-        interpolation_rule: str,
-        bars_frequency_min: int,
-        calendar: str,
-        last_observation: Union[None, pd.Series] = None,
+    bars_df: pd.DataFrame,
+    interpolation_rule: str,
+    bars_frequency_min: int,
+    calendar: str,
+    last_observation: None | pd.Series = None,
 ) -> pd.DataFrame:
     """
     Interpolates intraday bars based on the given parameters. Fills in missing data points in intraday bar data in case of gaps.
@@ -398,7 +399,7 @@ def interpolate_intraday_bars(
 
     def sanitize_today_update(x: pd.DataFrame, date_range, day):
         # normalize to UTC for consistent “today” comparison
-        today_utc =datetime.datetime.utcnow()
+        today_utc = datetime.datetime.utcnow()
         day_utc = day
 
         if day.date() == today_utc.date():
@@ -410,7 +411,7 @@ def interpolate_intraday_bars(
 
     def rebase_withoutnan_fill(x, trade_starts, trade_ends):
         date_range = build_daily_range_from_schedule(trade_starts, trade_ends)
-        date_range = sanitize_today_update(x=x, date_range=date_range,day=trade_starts)
+        date_range = sanitize_today_update(x=x, date_range=date_range, day=trade_starts)
         x = x.reindex(date_range)
         return x
 
@@ -421,7 +422,7 @@ def interpolate_intraday_bars(
         x["interpolated"] = False
         if not is_start_of_day:
             date_range = build_daily_range_from_schedule(trade_starts, trade_ends)
-            date_range = sanitize_today_update(x=x, date_range=date_range,day=trade_starts)
+            date_range = sanitize_today_update(x=x, date_range=date_range, day=trade_starts)
             try:
                 x = x.reindex(date_range)
 
@@ -459,23 +460,25 @@ def interpolate_intraday_bars(
     try:
         # because index are closes the greates value should be the open time of the last close to do not extra interpolate
 
-        restricted_schedule = calendar_instance.schedule(bars_df.index.min(),
-                                                         bars_df.iloc[-1]["open_time"])  # This needs to be faster
+        restricted_schedule = calendar_instance.schedule(
+            bars_df.index.min(), bars_df.iloc[-1]["open_time"]
+        )  # This needs to be faster
     except Exception as e:
         raise e
 
-    bars_df = bars_df[~bars_df.index.duplicated(keep='first')]  # todo: remove uncessary with indices.
+    bars_df = bars_df[
+        ~bars_df.index.duplicated(keep="first")
+    ]  # todo: remove uncessary with indices.
 
     full_index = bars_df.index.union(restricted_schedule.set_index("market_open").index).union(
-        restricted_schedule.set_index("market_close").index)
-
+        restricted_schedule.set_index("market_close").index
+    )
 
     bars_df = bars_df.reindex(full_index)
 
     bars_df["trade_day"] = bars_df.index
     bars_df["trade_day"] = pd.to_datetime(bars_df.index, utc=True).normalize()
 
-
     groups = bars_df.groupby("trade_day")
     interpolated_data = []
 
@@ -502,13 +505,15 @@ def interpolate_intraday_bars(
             trade_ends = next_day - datetime.timedelta(minutes=1)
 
         if interpolation_rule == "None":
-            tmp_df = rebase_withoutnan_fill(group_df, trade_starts=trade_starts, trade_ends=trade_ends)
+            tmp_df = rebase_withoutnan_fill(
+                group_df, trade_starts=trade_starts, trade_ends=trade_ends
+            )
         elif interpolation_rule == "ffill":
             tmp_df = rebase_with_forward_fill(
                 group_df,
                 trade_starts=trade_starts,
                 last_observation=last_observation,
-                trade_ends=trade_ends
+                trade_ends=trade_ends,
             )
             if last_observation is None:
                 tmp_df = tmp_df.bfill()
@@ -535,35 +540,39 @@ class InterpolatedPrices(DataNode):
     """
     Handles interpolated prices for assets.
     """
+
     OFFSET_START = datetime.datetime(2017, 7, 20).replace(tzinfo=pytz.utc)
-    _ARGS_IGNORE_IN_STORAGE_HASH=["asset_category_unique_id", "asset_list"]
+    _ARGS_IGNORE_IN_STORAGE_HASH = ["asset_category_unique_id", "asset_list"]
+
     def __init__(
-            self,
-            bar_frequency_id: str,
-            intraday_bar_interpolation_rule: str,
-            asset_category_unique_id: Optional[str] = None,
-            upsample_frequency_id: Optional[str] = None,
-            asset_list: List = None,
-            translation_table_unique_id: Optional[str] = None,
-            source_bars_data_node:Optional[DataNode] = None,
-            *args,
-            **kwargs
+        self,
+        bar_frequency_id: str,
+        intraday_bar_interpolation_rule: str,
+        asset_category_unique_id: str | None = None,
+        upsample_frequency_id: str | None = None,
+        asset_list: list = None,
+        translation_table_unique_id: str | None = None,
+        source_bars_data_node: DataNode | None = None,
+        *args,
+        **kwargs,
     ):
         """
         Initializes the InterpolatedPrices object.
         """
-        assert "d" in bar_frequency_id or "m" in bar_frequency_id, f"bar_frequency_id={bar_frequency_id} should be 'd for days' or 'm for min'"
-        if source_bars_data_node is  None:
+        assert (
+            "d" in bar_frequency_id or "m" in bar_frequency_id
+        ), f"bar_frequency_id={bar_frequency_id} should be 'd for days' or 'm for min'"
+        if source_bars_data_node is None:
             if asset_category_unique_id is None:
-                assert asset_list is not None, f"asset_category_unique_id={asset_category_unique_id} should not be None or asset_list should be defined"
-
-
+                assert (
+                    asset_list is not None
+                ), f"asset_category_unique_id={asset_category_unique_id} should not be None or asset_list should be defined"
 
         self.asset_category_unique_id = asset_category_unique_id
         self.interpolator = UpsampleAndInterpolation(
             bar_frequency_id=bar_frequency_id,
             upsample_frequency_id=upsample_frequency_id,
-            intraday_bar_interpolation_rule=intraday_bar_interpolation_rule
+            intraday_bar_interpolation_rule=intraday_bar_interpolation_rule,
         )
         self.constructor_asset_list = asset_list
         bars_frequency_min = string_frequency_to_minutes(bar_frequency_id)
@@ -572,12 +581,14 @@ class InterpolatedPrices(DataNode):
         self.intraday_bar_interpolation_rule = intraday_bar_interpolation_rule
         self.bar_frequency_id = bar_frequency_id
         self.upsample_frequency_id = upsample_frequency_id
-        self.source_bars_data_node=source_bars_data_node
+        self.source_bars_data_node = source_bars_data_node
         # get the translation rules
         if source_bars_data_node is None:
             if translation_table_unique_id is None:
-                raise Exception(f"Translation table needs to be set")
-            translation_table = AssetTranslationTable.get(unique_identifier=translation_table_unique_id)
+                raise Exception("Translation table needs to be set")
+            translation_table = AssetTranslationTable.get(
+                unique_identifier=translation_table_unique_id
+            )
 
             self.bars_ts = WrapperDataNode(translation_table=translation_table)
         else:
@@ -585,9 +596,8 @@ class InterpolatedPrices(DataNode):
 
         super().__init__(*args, **kwargs)
 
-
     def dependencies(self):
-        return {"bars_ts":self.bars_ts}
+        return {"bars_ts": self.bars_ts}
 
     def get_string_frequency_to_minutes(self):
         return string_frequency_to_minutes(self.bar_frequency_id)
@@ -608,8 +618,8 @@ class InterpolatedPrices(DataNode):
             self.local_persist_manager.protect_from_deletion()
 
     def _transform_raw_data_to_upsampled_df(
-            self,
-            raw_data_df: pd.DataFrame,
+        self,
+        raw_data_df: pd.DataFrame,
     ) -> pd.DataFrame:
         """
         Transforms raw data into an upsampled dataframe.
@@ -617,8 +627,10 @@ class InterpolatedPrices(DataNode):
         upsampled_df = []
 
         # TODO this should be a helper function
-        unique_identifier_range_map =self.update_statistics.get_update_range_map_great_or_equal()
-        full_last_observation = self.get_ranged_data_per_asset  (range_descriptor=unique_identifier_range_map)
+        unique_identifier_range_map = self.update_statistics.get_update_range_map_great_or_equal()
+        full_last_observation = self.get_ranged_data_per_asset(
+            range_descriptor=unique_identifier_range_map
+        )
         last_observation_map = {}
 
         for unique_identifier in raw_data_df["unique_identifier"].unique():
@@ -626,10 +638,13 @@ class InterpolatedPrices(DataNode):
                 last_observation_map[unique_identifier] = None
                 continue
 
-            if unique_identifier in full_last_observation.index.get_level_values("unique_identifier").to_list():
-                last_obs = full_last_observation.loc[(slice(None), unique_identifier), :].reset_index(
-                    ["unique_identifier"], drop=True
-                )
+            if (
+                unique_identifier
+                in full_last_observation.index.get_level_values("unique_identifier").to_list()
+            ):
+                last_obs = full_last_observation.loc[
+                    (slice(None), unique_identifier), :
+                ].reset_index(["unique_identifier"], drop=True)
                 last_obs.index.name = None
                 if "open_time" in last_obs.columns:
                     last_obs["open_time"] = pd.to_datetime(last_obs["open_time"], utc=True)
@@ -637,12 +652,14 @@ class InterpolatedPrices(DataNode):
             else:
                 last_observation_map[unique_identifier] = None
 
-        def multiproc_upsample(calendar, tmp_df, unique_identifier, last_observation, interpolator_kwargs):
+        def multiproc_upsample(
+            calendar, tmp_df, unique_identifier, last_observation, interpolator_kwargs
+        ):
             interpolator = UpsampleAndInterpolation(**interpolator_kwargs)
             df = interpolator.get_interpolated_upsampled_bars(
                 calendar=calendar,
                 tmp_df=tmp_df,
-                last_observation=last_observation_map[unique_identifier]
+                last_observation=last_observation_map[unique_identifier],
             )
             df["unique_identifier"] = unique_identifier
             return df
@@ -671,9 +688,10 @@ class InterpolatedPrices(DataNode):
                         bar_frequency_id=self.bar_frequency_id,
                         upsample_frequency_id=self.upsample_frequency_id,
                         intraday_bar_interpolation_rule=self.intraday_bar_interpolation_rule,
-                    )
+                    ),
                 )
-                for unique_identifier, tmp_df in raw_data_df.groupby("unique_identifier") if tmp_df.shape[0] > 0
+                for unique_identifier, tmp_df in raw_data_df.groupby("unique_identifier")
+                if tmp_df.shape[0] > 0
             )
 
         upsampled_df = [d for d in upsampled_df if len(d) > 0]  # Remove empty dataframes
@@ -686,7 +704,9 @@ class InterpolatedPrices(DataNode):
         max_end_time = max(max_value_per_asset.keys())
         self.logger.info(
             "Upsampled window aligned: min_end=%s (asset=%s), max_end=%s",
-            min_end_time.isoformat(), max_value_per_asset[min_end_time], max_end_time.isoformat()
+            min_end_time.isoformat(),
+            max_value_per_asset[min_end_time],
+            max_end_time.isoformat(),
         )
         upsampled_df = pd.concat(upsampled_df, axis=0)
         # upsampled_df = upsampled_df[upsampled_df.index <= min_max]
@@ -707,13 +727,16 @@ class InterpolatedPrices(DataNode):
         """
         unique_identifier_range_map = self.update_statistics.get_update_range_map_great_or_equal()
 
-
-        raw_data_df = self.bars_ts.get_ranged_data_per_asset(range_descriptor=unique_identifier_range_map)
+        raw_data_df = self.bars_ts.get_ranged_data_per_asset(
+            range_descriptor=unique_identifier_range_map
+        )
         if raw_data_df.empty:
             self.logger.info("No new data to interpolate")
             return pd.DataFrame()
 
-        upsampled_df = self._transform_raw_data_to_upsampled_df(raw_data_df.reset_index(["unique_identifier"]))
+        upsampled_df = self._transform_raw_data_to_upsampled_df(
+            raw_data_df.reset_index(["unique_identifier"])
+        )
         return upsampled_df
 
     def get_asset_list(self):
@@ -724,7 +747,7 @@ class InterpolatedPrices(DataNode):
             return self.bars_ts.get_asset_list()
 
         if self.constructor_asset_list is not None:
-            asset_list= self.constructor_asset_list
+            asset_list = self.constructor_asset_list
         else:
             asset_category = AssetCategory.get(unique_identifier=self.asset_category_unique_id)
             asset_list = Asset.filter(id__in=asset_category.assets)
@@ -735,18 +758,22 @@ class InterpolatedPrices(DataNode):
         """
         Updates the series from the source based on the latest value.
         """
-        self.asset_calendar_map = {a.unique_identifier: a.get_calendar() for a in self.update_statistics.asset_list}
+        self.asset_calendar_map = {
+            a.unique_identifier: a.get_calendar() for a in self.update_statistics.asset_list
+        }
         prices = self.get_upsampled_data()
         if prices.shape[0] == 0:
             return pd.DataFrame()
 
         if self.update_statistics.is_empty() == False:
-            TARGET_COLS = ['open', 'close', 'high', 'low', 'volume', 'open_time']
+            TARGET_COLS = ["open", "close", "high", "low", "volume", "open_time"]
             assert prices[[c for c in prices.columns if c in TARGET_COLS]].isnull().sum().sum() == 0
 
         prices = self.update_statistics.filter_df_by_latest_value(prices)
 
-        duplicates_exist = prices.reset_index().duplicated(subset=["time_index", "unique_identifier"]).any()
+        duplicates_exist = (
+            prices.reset_index().duplicated(subset=["time_index", "unique_identifier"]).any()
+        )
         if duplicates_exist:
             raise Exception()
 
@@ -758,18 +785,26 @@ class InterpolatedPrices(DataNode):
             self.logger.warning("trade_count not calculated in prices, set to NaN")
             prices["trade_count"] = np.nan
 
-        prices = prices[['open_time', 'open', 'high', 'low', 'close', 'volume', 'trade_count', 'vwap', 'interpolated']]
+        prices = prices[
+            [
+                "open_time",
+                "open",
+                "high",
+                "low",
+                "close",
+                "volume",
+                "trade_count",
+                "vwap",
+                "interpolated",
+            ]
+        ]
         return prices
 
+
 class ExternalPrices(DataNode):
 
     def __init__(
-            self,
-            artifact_name: str,
-            bucket_name: str,
-            asset_category_unique_id,
-            *args,
-            **kwargs
+        self, artifact_name: str, bucket_name: str, asset_category_unique_id, *args, **kwargs
     ):
         self.artifact_name = artifact_name
         self.bucket_name = bucket_name
@@ -786,6 +821,7 @@ class ExternalPrices(DataNode):
 
     def update(self) -> pd.DataFrame:
         from mainsequence.client.models_tdag import Artifact
+
         source_artifact = Artifact.get(bucket__name=self.bucket_name, name=self.artifact_name)
         prices_source = pd.read_csv(source_artifact.content)
 
@@ -795,13 +831,13 @@ class ExternalPrices(DataNode):
             "price",
         ]
         prices_source = prices_source[expected_cols].copy()
-        prices_source["time_index"] = pd.to_datetime(
-            prices_source["time_index"], utc=True
-        )
+        prices_source["time_index"] = pd.to_datetime(prices_source["time_index"], utc=True)
 
         # convert figis in source data
         for asset in self.update_statistics.asset_list:
-            prices_source.loc[prices_source["figi"] == asset.figi, "unique_identifier"] = asset.unique_identifier
+            prices_source.loc[prices_source["figi"] == asset.figi, "unique_identifier"] = (
+                asset.unique_identifier
+            )
 
         prices_source.set_index(["time_index", "unique_identifier"], inplace=True)
         prices = self.update_statistics.filter_df_by_latest_value(prices_source)
